package org.apache.bcel.util;

package .tmp;

import java.io.IOException;
import java.io.InputStream;
import org.apache.bcel.classfile.ClassParser;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.util.ClassPath;
import org.apache.bcel.util.Repository;

abstract class AbstractClassPathRepository implements Repository {
  private final ClassPath classPath;
  
  AbstractClassPathRepository(ClassPath classPath) {
    this.classPath = classPath;
  }
  
  public abstract void clear();
  
  public abstract JavaClass findClass(String paramString);
  
  public ClassPath getClassPath() {
    return this.classPath;
  }
  
  public JavaClass loadClass(Class<?> clazz) throws ClassNotFoundException {
    String className = clazz.getName();
    JavaClass repositoryClass = findClass(className);
    if (repositoryClass != null)
      return repositoryClass; 
    String name = className;
    int i = name.lastIndexOf('.');
    if (i > 0)
      name = name.substring(i + 1); 
    try (InputStream clsStream = clazz.getResourceAsStream(name + ".class")) {
      return loadClass(clsStream, className);
    } catch (IOException e) {
      return null;
    } 
  }
  
  private JavaClass loadClass(InputStream inputStream, String className) throws ClassNotFoundException {
    try {
      if (inputStream != null) {
        ClassParser parser = new ClassParser(inputStream, className);
        JavaClass clazz = parser.parse();
        storeClass(clazz);
        return clazz;
      } 
    } catch (IOException e) {
      throw new ClassNotFoundException("Exception while looking for class " + className + ": " + e, e);
    } 
    throw new ClassNotFoundException("ClassRepository could not load " + className);
  }
  
  public JavaClass loadClass(String className) throws ClassNotFoundException {
    if (className == null || className.isEmpty())
      throw new IllegalArgumentException("Invalid class name " + className); 
    className = Utility.pathToPackage(className);
    JavaClass clazz = findClass(className);
    if (clazz != null)
      return clazz; 
    try (InputStream inputStream = this.classPath.getInputStream(className)) {
      return loadClass(inputStream, className);
    } catch (IOException e) {
      throw new ClassNotFoundException("Exception while looking for class " + className + ": " + e, e);
    } 
  }
  
  public abstract void removeClass(JavaClass paramJavaClass);
  
  public abstract void storeClass(JavaClass paramJavaClass);
}
